from collective.contentsections.pages.base import IPage
from collective.contentsections.pages.base import Page
from plone.dexterity.content import Container
from zope.interface import implementer


class IBasicPage(IPage):
    """BasicPage schema"""


@implementer(IBasicPage)
class BasicPage(Page):
    """BasicPage class"""



from collective.contentsections import _
from collective.contentsections.sections.base import IBaseLinksSection
from collective.contentsections.sections.base import Section
from zope.interface import implementer


class IFilesSection(IBaseLinksSection):
    """FilesSection schema"""


@implementer(IFilesSection)
class FilesSection(Section):
    """FilesSection content type"""



from collective.contentsections.sections.base import BaseLinksSectionView
from collective.contentsections.sections.base import ISection
from plone import api


class LinksSectionView(BaseLinksSectionView):
    """LinksSection view"""

    def items(self):
        lead_image_scale = self.item_lead_image_scale
        brains = api.content.find(context=self.context, depth=1, portal_type="Link")
        results = [
            {
                "title": brain.Title,
                "description": brain.Description,
                "url": brain.getRemoteUrl,
                "lead_image_url": f"{brain.getURL()}/@@images/image/{lead_image_scale}",
                "effective_date": None,
                "tags": brain.Subject,
            }
            for brain in brains
        ]
        return results



from plone.app.z3cform.widget import SelectFieldWidget

from .base import BaseLinksSectionView
from .base import IBaseLinksSection
from .base import ISection
from .base import Section
from .base import SectionView
from .cards.content import CardsSection
from .cards.content import ICardsSection
from .collection.content import CollectionSection
from .collection.content import ICollectionSection
from .files.content import FilesSection
from .files.content import IFilesSection
from .html.content import HTMLSection
from .html.content import IHTMLSection
from .images.content import IImagesSection
from .images.content import ImagesSection
from .links.content import ILinksSection
from .links.content import LinksSection
from .selection.content import ISelectionSection
from .selection.content import SelectionSection
from .text.content import ITextSection
from .text.content import TextSection



# -*- coding: utf-8 -*-
from collective.contentsections import _
from plone import schema
from plone.autoform.interfaces import IFormFieldProvider
from plone.dexterity.interfaces import IDexterityContent
from plone.supermodel import model
from zope.component import adapter
from zope.interface import Interface
from zope.interface import implementer
from zope.interface import provider


class ILeadIcon(Interface):
    "Marker interface"


@provider(IFormFieldProvider)
class ILeadIconBehavior(model.Schema):

    icon = schema.Choice(
        title=_("Lead Icon"),
        required=False,
        vocabulary="collective.contentsections.LeadIcons",
    )


@implementer(ILeadIconBehavior)
@adapter(IDexterityContent)
class LeadIcon(object):
    def __init__(self, context):
        self.context = context

    @property
    def icon(self):
        return self.context.icon

    @icon.setter
    def icon(self, value):
        self.context.icon = value



# -*- coding: utf-8 -*-

import uuid

from plone.app.content.interfaces import INameFromTitle
from zope.component import adapter
from zope.interface import Interface
from zope.interface import implementer


class INameFromUUID(Interface):
    """Marker interface to enable 'Name from uuid' behavior"""


@implementer(INameFromTitle)
@adapter(INameFromUUID)
class NameFromUUID(object):
    def __init__(self, context):
        self.context = context

    @property
    def title(self):
        prefix = self.context.portal_type.split(".")[-1].lower()
        return f"{prefix}-{uuid.uuid4().hex}"



from collective.contentsections.pages import IPage
from collective.contentsections.sections import ICardsSection
from collective.contentsections.sections import ISection
from collective.contentsections.sections import ITextSection
from plone import api
from plone.app.contenttypes.behaviors.richtext import IRichText
from plone.app.contenttypes.indexers import SearchableText
from plone.app.contenttypes.indexers import _unicode_save_string_concat
from plone.indexer.decorator import indexer
from Products.CMFPlone.utils import base_hasattr


def get_title_and_description_terms(obj):
    terms = []
    if not obj.hide_title:
        # Index visible title
        terms.append(obj.title)
    if base_hasattr(obj, "description") and obj.description:
        # Index description
        terms.append(obj.description)
    return terms


def get_elements_title_and_description_terms(section):
    terms = []
    brains = api.content.find(context=section, depth=1, object_provides=ISection)
    for b in brains:
        terms.append(b.Title)
        terms.append(b.Description)

    return terms


@indexer(ISection)
def get_section_searchabletext(section):
    terms = get_title_and_description_terms(section)
    terms.extend(get_elements_title_and_description_terms(section))
    return " ".join(terms)


@indexer(ITextSection)
def get_textsection_searchabletext(section):
    terms = get_title_and_description_terms(section)
    transforms = api.portal.get_tool("portal_transforms")
    text = IRichText(section).text
    raw = text.raw
    plain = transforms.convertTo("text/plain", raw, mimetype=text.mimeType).getData().strip()
    terms.append(plain)
    return " ".join(terms)


@indexer(ICardsSection)
def get_cardssection_searchabletext(section):
    terms = get_title_and_description_terms(section)
    if section.cards:
        for card in section.cards:
            terms.append(card["title"])
            terms.append(card["subtitle"])
            terms.append(card["description"])
    return " ".join(terms)


@indexer(IPage)
def get_page_searchabletext(page):
    """Compute SearchableText of IPage with SearchableText of its ISection"""
    terms = [_unicode_save_string_concat(SearchableText(page))]
    catalog = api.portal.get_tool("portal_catalog")
    brains = api.content.find(context=page, depth=1, object_provides=ISection)
    for brain in brains:
        indexes = catalog.getIndexDataForRID(brain.getRID())
        searchable_terms = indexes.get("SearchableText") or []
        terms.extend(searchable_terms)
    return " ".join(terms)



from Acquisition import aq_inner
from collective.contentsections import _
from collective.contentsections.sections import ISection
from plone import api
from plone.app.contenttypes.browser.full_view import FullViewItem
from plone.dexterity.content import Container
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope.interface import Interface
from zope.interface import implementer


class IPage(Interface):
    """Shared base marker interface and schema for Pages"""


@implementer(IPage)
class Page(Container):
    """Shared base class for Pages"""

    def canSetDefaultPage(self):
        return False


class PageView(BrowserView):
    """Page view"""

    @property
    def sections(self):
        section_brains = api.content.find(
            context=self.context,
            depth=1,
            object_provides=ISection,
            sort_on="getObjPositionInParent",
        )
        section_objects = [b.getObject() for b in section_brains]
        return section_objects


class PageSectionView(FullViewItem):
    """Page view item"""


class PageTemplateView(BrowserView):
    def __call__(self):
        return ViewPageTemplateFile(self.template_name)

    @property
    def template_name(self):
        return "base_page_view.pt"

    @property
    def macros(self):
        return ViewPageTemplateFile(self.template_name).macros



from collective.contentsections import _
from collective.contentsections.sections.base import IBaseLinksSection
from collective.contentsections.sections.base import Section
from plone import schema
from plone.app.vocabularies.catalog import CatalogSource
from plone.app.z3cform.widget import RelatedItemsFieldWidget
from plone.autoform import directives
from plone.supermodel import model
from z3c.relationfield.schema import RelationChoice
from zope.interface import implementer


class ICollectionSection(IBaseLinksSection):
    """CollectionSection schema"""

    collection = RelationChoice(
        title=_(u"Collection"),
        source=CatalogSource(),
    )
    items_number = schema.Int(
        title=_(u"Number of items to display"),
        min=1,
        max=12,
    )
    collection_link_text = schema.TextLine(
        title=_(u"Text for the link to the collection"),
        required=False,
        default="",
        missing_value="",
    )

    directives.widget(
        "collection",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={"selectableTypes": ["Collection"], "favorites": []},
    )

    directives.order_after(collection="hide_title")
    directives.order_after(items_number="collection")


@implementer(ICollectionSection)
class CollectionSection(Section):
    """CollectionSection content type"""



from operator import itemgetter

from collective.contentsections import _
from plone import api
from plone.registry.interfaces import IRegistry
from zope.component import getUtility
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary

IMAGE_ALIGNMENTS = {
    "left": _("Left"),
    "right": _("Right"),
    "bottom": _("Bottom"),
    "top": _("Top"),
}
COLUMN_WIDTHS = {
    0: "",
    3: _("1/4"),
    4: _("1/3"),
    6: _("1/2"),
    8: _("2/3"),
    9: _("3/4"),
    12: _("1/1"),
}
COLUMN_ALIGNMENTS = {
    "start": _("Start"),
    "center": _("Center"),
    "end": _("End"),
}


class VocabularyFactory:
    def __init__(self, terms):
        self.terms = terms

    def __call__(self, context=None):
        terms = [SimpleTerm(value=value, token=str(value), title=title) for value, title in self.terms.items()]
        return SimpleVocabulary(terms)


ImageAlignmentsVocabulary = VocabularyFactory(IMAGE_ALIGNMENTS)
ColumnWidthsVocabulary = VocabularyFactory(COLUMN_WIDTHS)
ColumnAlignmentsVocabulary = VocabularyFactory(COLUMN_ALIGNMENTS)


class IconsVocabularyFactory:
    def __init__(self, prefix):
        self.prefix = prefix

    def __call__(self, context=None):
        registry = getUtility(IRegistry)
        items = [
            (key.replace(self.prefix, ""), record.field.title)
            for key, record in registry.records.items()
            if key.startswith(self.prefix)
        ]
        items.sort(key=itemgetter(1))
        terms = [SimpleTerm(value=value, token=str(value), title=title) for value, title in items]
        return SimpleVocabulary(terms)


LeadIconsVocabulary = IconsVocabularyFactory("plone.icon.")



from collective.contentsections import _
from collective.contentsections.sections.base import IBaseLinksSection
from collective.contentsections.sections.base import Section
from zope.interface import implementer


class IImagesSection(IBaseLinksSection):
    """ImagesSection schema"""


@implementer(IImagesSection)
class ImagesSection(Section):
    """ImagesSection content type"""



from collective.contentsections import _
from collective.contentsections.sections import SectionView
from plone import api


class CardsSectionView(SectionView):
    """Cards Section view"""

    @property
    def items(self):
        results = []
        for card in self.context.cards:
            relation_uid = card.get("relation_uid")
            remote_url = card.get("remote_url")
            if relation_uid:  # relation takes precedence over remote_url
                relation = api.content.get(UID=relation_uid) if relation_uid else None
                relation_link_url = relation.absolute_url() if relation else None
            elif remote_url:
                relation_link_url = remote_url
            else:
                relation_link_url = None

            results.append(
                {
                    "icon": card["icon"],
                    "title": card["title"],
                    "subtitle": card["subtitle"],
                    "description": card["description"],
                    "relation_link_url": relation_link_url,
                }
            )
        return results

    @property
    def relation_link_text(self):
        text = self.context.relation_link_text
        return text if text else _("More information")



from collective.contentsections import _
from collective.contentsections.sections.base import IBaseLinksSection
from collective.contentsections.sections.base import Section
from plone.app.vocabularies.catalog import StaticCatalogVocabulary
from plone.app.z3cform.widget import AjaxSelectFieldWidget
from plone.autoform import directives
from z3c.relationfield.schema import RelationChoice
from z3c.relationfield.schema import RelationList
from zope.interface import implementer


class ISelectionSection(IBaseLinksSection):
    """SelectionSection schema"""

    relations = RelationList(
        title=_(u"Items"),
        value_type=RelationChoice(
            title=_(u"Item"),
            vocabulary=StaticCatalogVocabulary({}),
        ),
    )

    directives.widget(
        "relations",
        AjaxSelectFieldWidget,
        vocabulary=StaticCatalogVocabulary({}, title_template="{brain.Type}: {brain.Title} at {path}"),
        pattern_options={
            "placeholder": _("Select items"),
            "minimumInputLength": 3,
            "ajax": {"quietMillis": 300},
        },
    )
    directives.order_after(relations="hide_title")


@implementer(ISelectionSection)
class SelectionSection(Section):
    """SelectionSection content type"""



from collective.contentsections.sections.base import BaseLinksSectionView


class CollectionSectionView(BaseLinksSectionView):
    """Collection Section view"""

    def items(self):
        lead_image_scale = self.item_lead_image_scale
        brains = self.context.collection.to_object.results(
            batch=False,
            brains=True,
            limit=self.context.items_number,
        )
        results = [
            {
                "title": brain.Title,
                "description": brain.Description,
                "url": brain.getURL(),
                "lead_image_url": f"{brain.getURL()}/@@images/image/{lead_image_scale}",
                "effective_date": brain.effective,
                "start_date": brain.start,
                "end_date": brain.end,
                "tags": brain.Subject,
            }
            for brain in brains
        ]
        return results

    @property
    def more_link_url(self):
        return self.context.collection.to_object.absolute_url()

    @property
    def more_link_text(self):
        return self.context.collection_link_text



from collective.contentsections import _
from collective.contentsections.sections.base import IBaseLinksSection
from collective.contentsections.sections.base import Section
from zope.interface import implementer


class ILinksSection(IBaseLinksSection):
    """LinksSection schema"""


@implementer(ILinksSection)
class LinksSection(Section):
    """LinksSection content type"""



from collective.contentsections import _
from collective.contentsections.sections.base import ISection
from collective.contentsections.sections.base import Section
from collective.z3cform.datagridfield.datagridfield import DataGridFieldFactory
from collective.z3cform.datagridfield.row import DictRow
from plone import schema
from plone.app.vocabularies.catalog import StaticCatalogVocabulary
from plone.app.z3cform.widget import AjaxSelectFieldWidget
from plone.app.z3cform.widget import SelectFieldWidget
from plone.autoform import directives
from plone.supermodel import model
from z3c.relationfield.schema import RelationChoice
from zope.interface import Interface
from zope.interface import implementer


class ICard(Interface):
    """Card schema"""

    title = schema.TextLine(
        title=_("Title"),
        required=False,
        default="",
        missing_value="",
    )
    subtitle = schema.TextLine(
        title=_("Subtitle"),
        required=False,
        default="",
        missing_value="",
    )
    description = schema.Text(
        title=_("Text"),
        required=False,
        default="",
        missing_value="",
    )
    icon = schema.Choice(
        title=_("Icon"),
        vocabulary="collective.contentsections.LeadIcons",
        required=False,
        default="",
        missing_value="",
    )
    relation_uid = schema.Choice(
        title=_("Selection"),
        vocabulary=StaticCatalogVocabulary({}),
        required=False,
    )
    remote_url = schema.TextLine(
        title=_("Remote url"),
        required=False,
        default="",
        missing_value="",
    )

    directives.widget("title", placeholder=_("Card title"))
    directives.widget("subtitle", placeholder=_("Card subtitle"))
    directives.widget("icon", SelectFieldWidget, pattern_options={"placeholder": _("Select an icon")})
    directives.widget("remote_url", placeholder=_("Remote url"))

    # SEE https://training.plone.org/5/mastering-plone/relations.html?highlight=catalogsource#using-different-widgets-for-relations
    directives.widget(
        "relation_uid",
        AjaxSelectFieldWidget,
        vocabulary=StaticCatalogVocabulary({}, title_template="{brain.Type}: {brain.Title} at {path}"),
        pattern_options={
            "placeholder": _("Select a related content"),
            "minimumInputLength": 3,
            "ajax": {"quietMillis": 300},
        },
    )


class ICardsSection(ISection):
    """CardsSection schema"""

    cards = schema.List(
        title=_("Cards"),
        value_type=DictRow(title=_("Card"), schema=ICard),
        min_length=1,
        missing_value=[],
    )
    group_size = schema.Choice(
        title=_("Group size"),
        values=[1, 2, 3, 4],
        default=3,
    )
    relation_link_text = schema.TextLine(
        title=_(u"Text for the link to the related content"),
        required=False,
    )

    directives.widget("cards", DataGridFieldFactory, allow_reorder=True)
    directives.order_before(group_size="is_full_width")


@implementer(ICardsSection)
class CardsSection(Section):
    """CardsSection content type"""



from .base import IPage
from .base import Page
from .base import PageView
from .basic.content import BasicPage
from .basic.content import IBasicPage
from .event.content import EventPage
from .event.content import IEventPage



from collective.contentsections import _
from collective.contentsections.sections.base import ISection
from collective.contentsections.sections.base import Section
from plone import schema
from zope.interface import implementer


class IHTMLSection(ISection):
    """HTMLSection schema"""

    html = schema.SourceText(
        title=_(u"HTML"),
    )


@implementer(IHTMLSection)
class HTMLSection(Section):
    """HTMLSection content type"""



from collective.contentsections.sections.base import BaseLinksSectionView
from collective.contentsections.sections.base import ISection
from collective.contentsections.sections.base import SectionView
from plone import api


class ImagesSectionView(BaseLinksSectionView):
    """ImagesSection view"""

    def items(self):
        lead_image_scale = self.item_lead_image_scale
        brains = api.content.find(context=self.context, depth=1, portal_type="Image")
        results = [
            {
                "title": brain.Title,
                "description": brain.Description,
                "url": None,  # TODO
                "lead_image_url": f"{brain.getURL()}/@@images/image/{lead_image_scale}",
                "effective_date": brain.effective,
                "tags": [],
            }
            for brain in brains
        ]
        return results


class ImagesSectionGalleryView(SectionView):
    """ImagesSection gallery view"""

    def items(self):
        brains = api.content.find(context=self.context, depth=1, portal_type="Image")
        results = [
            {
                "title": brain.Title,
                "description": brain.Description,
                "full_image_url": f"{brain.getURL()}/@@images/image/huge",
                "preview_image_url": f"{brain.getURL()}/@@images/image/preview",
            }
            for brain in brains
        ]
        return results



from collective.contentsections import _
from collective.contentsections.sections.base import ISection
from collective.contentsections.sections.base import Section
from plone import schema
from plone.autoform import directives
from zope.interface import implementer


class ITextSection(ISection):
    """TextSection schema"""

    lead_image_alignment = schema.Choice(
        title=_(u"Lead image alignment"),
        vocabulary="collective.contentsections.ImageAlignments",
        default="right",
    )
    lead_image_scale = schema.Choice(
        title=_(u"Lead image scale"),
        vocabulary="plone.app.vocabularies.ImagesScales",
        default="preview",
    )

    directives.order_before(lead_image_alignment="IVersionable.changeNote")
    directives.order_after(lead_image_scale="lead_image_alignment")


@implementer(ITextSection)
class TextSection(Section):
    """TextSection content type"""



from collective.contentsections.sections.base import BaseLinksSectionView
from plone import api


class SelectionSectionView(BaseLinksSectionView):
    """Selection Section view"""

    def items(self):
        lead_image_scale = self.item_lead_image_scale
        objects = [rel.to_object for rel in self.context.relations if not rel.isBroken()]
        results = [
            {
                "title": obj.title,
                "description": obj.description,
                "url": obj.absolute_url(),
                "lead_image_url": f"{obj.absolute_url()}/@@images/image/{lead_image_scale}",
                "effective_date": obj.effective if hasattr(obj, "effective") else None,
                "start_date": obj.start if hasattr(obj, "start") else None,
                "end_date": obj.start if hasattr(obj, "start") else None,
                "tags": obj.subjects,
            }
            for obj in objects
        ]
        return results



from collective.contentsections.pages.base import IPage
from collective.contentsections.pages.base import Page
from zope.interface import implementer


class IEventPage(IPage):
    """EventPage schema"""


@implementer(IEventPage)
class EventPage(Page):
    """EventPage class"""



from setuptools import find_packages
from setuptools import setup

long_description = "\n\n".join(
    [
        open("README.md").read(),
        open("CONTRIBUTORS.md").read(),
        open("CHANGES.md").read(),
    ]
)
setup(
    name="collective.contentsections",
    version="1.0.0a5",
    description="A block approach for Plone 6 Classic",
    long_description=long_description,
    long_description_content_type="text/markdown",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Environment :: Web Environment",
        "Framework :: Plone",
        "Framework :: Plone :: Addon",
        "Framework :: Plone :: 6.0",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Operating System :: OS Independent",
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
    ],
    keywords="Python Plone CMS",
    author="Sébastien Verbois",
    author_email="sebastien.verbois@gmail.be",
    url="https://github.com/sverbois/collective.contentsections",
    project_urls={
        "PyPI": "https://pypi.python.org/pypi/collective.contentsections",
        "Source": "https://github.com/sverbois/collective.contentsections",
        "Tracker": "https://github.com/sverbois/collective.contentsections/issues",
        # 'Documentation': 'https://collective.contentsections.readthedocs.io/en/latest/',
    },
    license="GPL version 2",
    packages=find_packages(where="src", exclude=["ez_setup"]),
    namespace_packages=["collective"],
    package_dir={"": "src"},
    include_package_data=True,
    zip_safe=False,
    python_requires=">=3.8",
    install_requires=[
        "setuptools",
        # -*- Extra requirements: -*-
        "plone.api",
        "plone.app.dexterity",
        "collective.z3cform.datagridfield",
    ],
    entry_points="""
    [z3c.autoinclude.plugin]
    target = plone
    """,
)



from collective.contentsections import _
from plone import schema
from plone.autoform import directives
from plone.dexterity.content import Container
from plone.namedfile.field import NamedBlobImage
from plone.supermodel import model
from Products.Five.browser import BrowserView
from zope.interface import implementer


class ISection(model.Schema):
    """Shared base marker interface and schema for Sections"""

    title = schema.TextLine(
        title=_("Section title"),
    )
    hide_title = schema.Bool(
        title=_("Hide section title in page"),
        default=False,
    )
    is_full_width = schema.Bool(
        title=_("Full width"),
        default=False,
    )
    background_image = NamedBlobImage(
        title=_("Background image"),
        required=False,
    )
    css_classes = schema.TextLine(
        title=_("CSS Classes"),
        required=False,
        default="",
        missing_value="",
    )

    model.fieldset(
        "layout",
        label="Layout",
        fields=["is_full_width", "background_image", "css_classes"],
    )


class IBaseLinksSection(ISection):
    """Shared base marker interface and schema for CollectionSection, SelectionSection and LinksSection"""

    hide_item_lead_images = schema.Bool(
        title=_("Hide item lead images"),
        default=False,
    )
    hide_item_descriptions = schema.Bool(
        title=_("Hide item descriptions"),
        default=False,
    )
    hide_item_publication_dates = schema.Bool(
        title=_("Hide item publication dates"),
        default=True,
    )
    group_size = schema.Choice(
        title=_("Group size"),
        values=[1, 2, 3, 4],
        default=3,
    )

    model.fieldset(
        "layout",
        fields=[
            "group_size",
        ],
    )
    directives.order_before(group_size="is_full_width")


@implementer(ISection)
class Section(Container):
    """Shared base class for Sections"""

    def canSetDefaultPage(self):
        return False


class SectionView(BrowserView):
    """Section view"""

    def __call__(self):
        page = self.context.__parent__
        url = f"{page.absolute_url()}#section-{self.context.id}"
        self.request.response.redirect(url)


class BaseLinksSectionView(SectionView):
    """Shared section view for CollectionSection, SelectionSection and LinksSection"""

    @property
    def items(self):
        return []

    @property
    def more_link_url(self):
        return None

    @property
    def more_link_text(self):
        return None

    @property
    def item_lead_image_scale(self):
        layout = self.context.getLayout()
        group_size = self.context.group_size
        cols = 12 / group_size
        if layout in ["carousel_view", "cards_view"]:
            if cols > 6:
                return "huge"
            elif cols > 3:
                return "large"
            else:
                return "preview"
        elif layout == "list_view":
            return "thumb"
        return "preview"


def reindex_parent_page(section, event):
    page = section.__parent__
    if page is not None:
        page.reindexObject()
